"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Mdoc = void 0;
const kmp_mdl_mdoc_1 = require("@sphereon/kmp-mdl-mdoc");
const crypto_1 = require("../../crypto");
const x509_1 = require("../../modules/x509");
const utils_1 = require("../../utils");
const MdocCoseCallbackService_1 = require("./MdocCoseCallbackService");
const MdocError_1 = require("./MdocError");
const MdocX509CallbackService_1 = require("./MdocX509CallbackService");
class Mdoc {
    constructor(buffer) {
        // TODO: CONVERSION FROM CBOR TO JSON AND BACK CURRENTLY NOT COMPLETELY WORKING THEREFORE WE STORE BOTH FOR NOW
        this.issuerSignedCbor = kmp_mdl_mdoc_1.com.sphereon.mdoc.data.device.IssuerSignedCbor.Companion.cborDecode(Int8Array.from(buffer));
        this.issuerSignedJson = this.issuerSignedCbor.toJson();
    }
    get docType() {
        // TODO: This will be a part of the { ... issuerSigned } structure
        return 'org.iso.18013.5.1.mDL';
    }
    // TODO: Use a different return type. Wait for sphereon
    get namespaces() {
        var _a;
        const namespaces = {};
        const entries = (_a = this.issuerSignedJson.nameSpaces) === null || _a === void 0 ? void 0 : _a.asJsMapView().entries();
        // eslint-disable-next-line no-constant-condition
        while (true) {
            const next = entries === null || entries === void 0 ? void 0 : entries.next();
            if (next === null || next === void 0 ? void 0 : next.done)
                break;
            if (!(next === null || next === void 0 ? void 0 : next.value)) {
                throw new MdocError_1.MdocError('Missing value in Mdoc');
            }
            // TODO: This is not completely working yet. The values are still not json if nested.
            // TODO: wait for sphereon to complete
            // Waiting for fix from sphereon
            const mdocDataItem = next.value[1];
            const mdocDataItemRecord = Object.fromEntries(mdocDataItem.map((dataItem) => [dataItem.elementIdentifier, dataItem.elementValue]));
            namespaces[next.value[0]] = mdocDataItemRecord;
        }
        return namespaces;
    }
    get jwaSignatureAlgorithm() {
        const alg = this.issuerSignedJson.issuerAuth.protectedHeader.alg;
        if (!alg) {
            throw new MdocError_1.MdocError(`Missing Signature Algorithm in Mdoc.`);
        }
        const jwaAlgorithm = kmp_mdl_mdoc_1.com.sphereon.crypto.SignatureAlgorithmMapping.Companion.toJose(alg)
            .value;
        if (!Object.values(crypto_1.JwaSignatureAlgorithm).includes(jwaAlgorithm)) {
            throw new MdocError_1.MdocError(`Invalid Signature Algorithm on MDoc Document. Alg '${alg}'`);
        }
        return jwaAlgorithm;
    }
    static fromIssuerSignedHex(hexEncodedMdoc) {
        return new Mdoc(utils_1.TypedArrayEncoder.fromHex(hexEncodedMdoc));
    }
    static fromIssuerSignedBase64Url(base64UrlEncodedMdoc) {
        const hexEncodedMdoc = utils_1.TypedArrayEncoder.fromBase64(base64UrlEncodedMdoc).toString('hex');
        return this.fromIssuerSignedHex(hexEncodedMdoc);
    }
    get issuerSignedHex() {
        return utils_1.TypedArrayEncoder.toHex(utils_1.Buffer.from(this.issuerSignedCbor.toCbor().cborEncode()));
    }
    get issuerSignedBase64UrlEncoded() {
        return utils_1.TypedArrayEncoder.toBase64URL(utils_1.Buffer.from(this.issuerSignedCbor.toCbor().cborEncode()));
    }
    async verify(agentContext, options) {
        var _a;
        const { trustedCertificates } = options !== null && options !== void 0 ? options : {};
        const cryptoServiceJS = kmp_mdl_mdoc_1.com.sphereon.crypto.CryptoServiceJS;
        // TODO: This way of of registering and working with the x509/cose services is subject to race-conditions
        // TODO: This is a known issue and beeing worked on by sphereon
        cryptoServiceJS.X509.register(new MdocX509CallbackService_1.MdocX509CallbackService(agentContext, trustedCertificates));
        cryptoServiceJS.COSE.register(new MdocCoseCallbackService_1.MdocCoseCallbackService(agentContext));
        const certificateChain = (_a = this.issuerSignedCbor.issuerAuth.toJson().unprotectedHeader) === null || _a === void 0 ? void 0 : _a.x5chain;
        if (!certificateChain) {
            return {
                isValid: false,
                error: `The issuer signed structure is missing the 'x5chain' property in the unprotected header.`,
            };
        }
        // TODO: CHECK why this is required
        const _leafCertificate = utils_1.TypedArrayEncoder.toBase64(utils_1.TypedArrayEncoder.fromBase64(certificateChain[0]));
        const leafCertificate = x509_1.X509Service.getLeafCertificate(agentContext, { certificateChain: [_leafCertificate] });
        const verificationResult = await kmp_mdl_mdoc_1.com.sphereon.mdoc.ValidationsJS.fromIssuerAuthAsync(this.issuerSignedCbor.issuerAuth, 
        // This key is used later on in the cose verification callback for signature verification
        { opts: kmp_mdl_mdoc_1.kotlin.collections.KtMap.fromJsMap(new Map([['publicKey', leafCertificate.publicKey]])) }, trustedCertificates);
        if (verificationResult.error) {
            return {
                isValid: false,
                error: verificationResult.verifications.join(' '),
            };
        }
        return { isValid: true };
    }
}
exports.Mdoc = Mdoc;
//# sourceMappingURL=Mdoc.js.map