"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MdocX509CallbackService = void 0;
const x509_1 = require("../x509");
const X509Certificate_1 = require("../x509/X509Certificate");
const X509Service_1 = require("../x509/X509Service");
const MdocError_1 = require("./MdocError");
/**
 * This class can be used for X509 validations.
 * Either have an instance per trustedCerts and verification invocation or use a single instance and provide the trusted certs in the method argument
 *
 * The class is also registered with the low-level mDL/mdoc Kotlin Multiplatform library
 * Next to the specific function for the library it exports a more powerful version of the same verification method as well
 */
class MdocX509CallbackService {
    constructor(agentContext, trustedCertificates) {
        this.agentContext = agentContext;
        this.getTrustedCerts = () => {
            return this.trustedCertificates;
        };
        const _trustedCertificates = trustedCertificates !== null && trustedCertificates !== void 0 ? trustedCertificates : agentContext.dependencyManager.resolve(x509_1.X509ModuleConfig).trustedCertificates;
        if (!_trustedCertificates) {
            throw new MdocError_1.MdocError('Missing trusted certificates for Mdoc validation.');
        }
        this.trustedCertificates = _trustedCertificates;
    }
    /**
     * This method is the implementation used within the mDL/Mdoc library
     */
    async verifyCertificateChainJS(chainDER) {
        if (!chainDER) {
            return {
                name: 'x509-verification invalid parameters',
                message: 'Missing ChainDER parameter when verifying the Certificate chain.',
                critical: true,
                error: true,
            };
        }
        try {
            const certificateChain = chainDER.map((value) => X509Certificate_1.X509Certificate.fromRawCertificate(new Uint8Array(value)).toString('base64url'));
            await X509Service_1.X509Service.validateCertificateChain(this.agentContext, { certificateChain });
            return {
                name: 'x509-verification',
                message: 'x509-chain successfully validated',
                critical: false,
                error: false,
            };
        }
        catch (error) {
            return {
                name: 'x509-verification',
                message: error instanceof Error
                    ? `Modoc x509 certificate chain validation failed. ${error.message}`
                    : 'An unknown error occurred during x509 certificate chain validation.',
                critical: true,
                error: true,
            };
        }
    }
}
exports.MdocX509CallbackService = MdocX509CallbackService;
//# sourceMappingURL=MdocX509CallbackService.js.map